import numpy as np
from sympy import *


def sphereMetric():
    inputsym = ['r', 'theta', 'phi']
    ginput = {}
    for i in inputsym:
        tmp = {}
        for j in inputsym:
            tmp[j] = '0';
        ginput[i] = tmp
    ginput['r']['r'] = '1';
    ginput['theta']['theta'] = 'r*r'
    ginput['phi']['phi'] = 'r*r*sin(theta)*sin(theta)'
    return ginput,inputsym
def orthMetric(n):
    inputsym = ['x'+str(i) for i in range(n)]
    ginput = {}
    for i in inputsym:
        tmp = {}
        for j in inputsym:
            tmp[j] = '0';
        ginput[i] = tmp
    for i in range(n):
        ginput[inputsym[i]][inputsym[i]]='1'
    return ginput, inputsym
def coord2MetricFromOrth(n,coordList,varlist):
    #i.e.   n=3;   coordList=['r*cos(theta)*cos(phi)','r*cos(theta)*sin(phi)','r*sin(theta)']
    #              varlist=['r','theta','phi']
    assert(n==len(coordList))
    assert(n==len(varlist))
    x = [0] * n
    sym={}
    for i in range(n):
        sym[varlist[i]] = i
        x[i] = Symbol(varlist[i])
    for i in range(n):
        coordList[i]=eval(changeVarName(coordList[i],varlist,x))
    g = {}
    for i in varlist:
        tmp = {}
        for j in varlist:
            tmp[j] = str(simplify(sum([diff(coordList[coord],x[sym[i]])*diff(coordList[coord],x[sym[j]]) for coord in range(n)])));
        g[i] = tmp


    return g,varlist


def ischar(ch):
    return ord(ch) >= 97 and ord(ch) <= 122
def changeVarName(s,varlist,x):
    cnt=0
    try:
        m=len(s)
    except TypeError:
        print('need to write metric in string')
    tmp=''
    flag=[True]*m
    for i in range(m):
        if flag[i]==False:
            continue
        breakj=False
        for j in range(i+1,m+1):
            for k in range(len(varlist)):
                if s[i:j] ==varlist[k]:
                    if (i==0 or (not ischar(s[i-1]))) and (j==m or (not ischar(s[j]))):
                        tmp=tmp+'x['+str(k)+']'
                        flag[i:j]= [False]*(j-i)
                        breakj=True
                        break
            if breakj==True:# or (j==m or (not ischar(s[j]))):
                break
        if breakj==False:
            tmp=tmp+s[i]
    # print(tmp)
    return tmp
def getChristSymb(ginput,coordinateName):

# coordinateName is the given axis name i.e. ['r', 'theta','phi']
# ginput is metric field i.e. ginput={'r': {'r': '1', 'theta': '0', 'phi': '0'},
#                           'theta': {'r': '0', 'theta': 'r*r', 'phi': '0'},
#                           'phi': {'r': '0', 'theta': '0', 'phi': 'r*r*sin(theta)*sin(theta)'}}
    n=len(coordinateName)
    x = [0] * n
    sym={};
    for i in range(n):
        sym[coordinateName[i]] = i
        x[i] = Symbol(coordinateName[i])
    g = Matrix([[eval(changeVarName(ginput[coordinateName[i]][coordinateName[j]], coordinateName,x)) for j in range(n)] for i in range(n)])
    g_inv = (g ** -1)
    g_deriv = np.array([[[diff(g[i, j], x[k]) for k in range(n)] for j in range(n)] for i in range(n)])
    # T=(g_deriv+np.transpose(g_deriv,(0,2,1))-np.transpose(g_deriv,(1,2,0)))
    # T=(np.dot(g_inv,np.transpose(T,(1,0,2))))/2
    T = ([[[0 for i in range(n)] for j in range(n)] for k in range(n)])
    for i in range(n):
        for j in range(n):
            for k in range(n):
                tmp = g_inv[i, :] * Matrix((g_deriv[:, j, k] + g_deriv[:, k, j] - g_deriv[j, k, :]) / 2)
                T[i][j][k] = tmp[0]

    return T
def getRiemannCurvature(T,symb):# T - christSymb n*n*n            symb   -    r, theta, phi
    n=len(symb)
    x = [0] * n
    for i in range(n):
        x[i] = Symbol(symb[i])
    R = ([[[[0 for m in range(n)] for i in range(n)] for j in range(n)] for k in range(n)])
    for u in range(n):
        for v in range(n):
            for d in range(n):
                for p in range(n):
                    tmp=sum([T[lam][d][u]*T[p][v][lam]-T[lam][d][v]*T[p][u][lam] for lam in range(n)])
                    R[u][v][d][p]=diff(T[p][u][d],x[v])-diff(T[p][v][d],x[u])+tmp
    print(1)
    return R
def getRici(T,symb):
    n = len(symb)
    x = [0] * n
    for i in range(n):
        x[i] = Symbol(symb[i])
    Rici=[[0 for i in range(n)] for j in range(n)]
    for u in range(n):
        for d in range(n):
            tmp=sum([diff(T[v][u][d],x[v])-diff(T[v][v][d],x[u]) for v in range(n)])
            Rici[u][d]=tmp+sum([sum([T[lam][u][d]*T[v][lam][v]-T[lam][v][d]*T[v][lam][u] for lam in range(n)]) for v in range(n)])
    return Rici
def listDepth(list):
    depth=0
    if type(list)!=type([]):
        return depth
    tmp=list[0]
    depth=depth+1
    while type(tmp)==type([]):
        tmp=tmp[0]
        depth=depth+1
    return depth
def list2Dict(list,symb,depth):
    # depth=listDepth(list)
    n=len(symb)
    if depth==1:
        return dict(zip(symb,list))
    else:
        return dict(zip(symb,[list2Dict(list[i],symb,depth-1) for i in range(n)]))
    return
# g,v=coord2MetricFromOrth(3,['r*cos(theta)*cos(phi)','r*cos(theta)*sin(phi)','r*sin(theta)'],['r','theta','phi'])
print('import util success')
print('''use   [g,symbol] = sphereMetric() to create a example metric
        ##symbol=['r','theta','phi']##
use T=getChristSymb(g,symbol) to get the christ tensor
use R=getRiemannCurvature(T,symbol) to get riemann curvature tensor
The input metric g should be given in dictionary form
''')




from tkinter import  *
root =Tk()
def key(event):
    print("pressed", repr(event.char))

def callback(event):
    print ("clicked at", event.x, event.y)

frame = Frame(root, width=100, height=100)
frame.bind("<Key>", key)
frame.bind("<Button-1>", callback)
frame.pack()

root.mainloop()



import tkinter as tk  # 使用Tkinter前需要先导入

# 第1步，实例化object，建立窗口window
window = tk.Tk()

# 第2步，给窗口的可视化起名字
window.title('My Window')

# 第3步，设定窗口的大小(长 * 宽)
window.geometry('500x300')  # 这里的乘是小x

# 第4步，在图形界面上创建 500 * 200 大小的画布并放置各种元素
canvas = tk.Canvas(window, bg='green', height=200, width=500)
# 说明图片位置，并导入图片到画布上
image_file = tk.PhotoImage(file='pic.png')  # 图片位置（相对路径，与.py文件同一文件夹下，也可以用绝对路径，需要给定图片具体绝对路径）
image = canvas.create_image(250, 0, anchor='n', image=image_file)  # 图片锚定点（n图片顶端的中间点位置）放在画布（250,0）坐标处
# 定义多边形参数，然后在画布上画出指定图形
x0, y0, x1, y1 = 100, 100, 150, 150
line = canvas.create_line(x0 - 50, y0 - 50, x1 - 50, y1 - 50)  # 画直线
oval = canvas.create_oval(x0 + 120, y0 + 50, x1 + 120, y1 + 50, fill='yellow')  # 画圆 用黄色填充
arc = canvas.create_arc(x0, y0 + 50, x1, y1 + 50, start=0, extent=180)  # 画扇形 从0度打开收到180度结束
rect = canvas.create_rectangle(330, 30, 330 + 20, 30 + 20)  # 画矩形正方形
canvas.pack()


# 第6步，触发函数，用来一定指定图形
def moveit():
    canvas.move(rect, 2, 2)  # 移动正方形rect（也可以改成其他图形名字用以移动一起图形、元素），按每次（x=2, y=2）步长进行移动


# 第5步，定义一个按钮用来移动指定图形的在画布上的位置
b = tk.Button(window, text='move item', command=moveit).pack()

# 第7步，主窗口循环显示
window.mainloop()


from mpl_toolkits.mplot3d import Axes3D
import matplotlib.pyplot as plt
from sympy import *
from math import *
from difdyf.util_func import *
import numpy as np

def plotFromCoord(coord,var,ranges,c='r',m='o'):# ranges: [range(l,h,step),range(l,h,step)...]
    assert(len(coord)==len(var) and len(var)==len(ranges))
    ndim=len(var)


    if ndim>3:
        print('dim should be lower than 4')
        return-1
    x = [0] * ndim
    sym = {}
    for i in range(ndim):
        sym[var[i]] = i
        x[i] = Symbol(var[i])
    for i in range(ndim):
        coord[i] = eval(changeVarName(coord[i], var, x))
    if ndim==3:
        x0=np.array([coord[0].subs([(x[0],y0),(x[1],y1),(x[2],y2)]) for y0 in ranges[0] for y1 in ranges[1] for y2 in ranges[2]],dtype=float)
        x1 = np.array([coord[1].subs([(x[0], y0), (x[1], y1), (x[2], y2)]) for y0 in ranges[0] for y1 in ranges[1] for y2 in
              ranges[2]],dtype=float)
        x2 = np.array([coord[2].subs([(x[0], y0), (x[1], y1), (x[2], y2)]) for y0 in ranges[0] for y1 in ranges[1] for y2 in
              ranges[2]],dtype=float)
        fig = plt.figure()
        ax = fig.add_subplot(111, projection='3d')
        ax.scatter(x0, x1, x2,c=c,marker=m)
        ax.set_xlabel('X Label')
        ax.set_ylabel('Y Label')
        ax.set_zlabel('Z Label')
        plt.show()

        # for y1 in ranges[0]:
        #     for y2 in ranges[1]:
        #         for y3 in ranges[3]:




    return 0
# g,v=coord2MetricFromOrth(3,['r*cos(theta)*cos(phi)','r*cos(theta)*sin(phi)','r*sin(theta)'],['r','theta','phi'])

















# import matplotlib.pyplot as plt
# from mpl_toolkits.mplot3d import Axes3D
# fig = plt.figure()
# ax = fig.add_subplot(111, projection='3d')
# plt.show()
####################################################### curve
# from mpl_toolkits.mplot3d import Axes3D
# import numpy as np
# import matplotlib.pyplot as plt
#
# fig = plt.figure()
# ax = fig.gca(projection='3d')
#
# # Prepare arrays x, y, z
# theta = np.linspace(-4 * np.pi, 4 * np.pi, 100)
# z = np.linspace(-2, 2, 100)
# r = z**2 + 1
# x = r * np.sin(theta)
# y = r * np.cos(theta)
#
# ax.plot(x, y, z, label='parametric curve')  #这里传入x, y, z的值
# ax.legend()
#
# plt.show()
############################################################### discreate points
from mpl_toolkits.mplot3d import Axes3D
import matplotlib.pyplot as plt
import numpy as np

# Fixing random state for reproducibility
np.random.seed(19680801)


def randrange(n, vmin, vmax):
    '''
    Helper function to make an array of random numbers having shape (n, )
    with each number distributed Uniform(vmin, vmax).
    '''
    return (vmax - vmin)*np.random.rand(n) + vmin

fig = plt.figure()
ax = fig.add_subplot(111, projection='3d')

n = 100

# For each set of style and range settings, plot n random points in the box
# defined by x in [23, 32], y in [0, 100], z in [zlow, zhigh].
# 迭代两次，分别绘制两种不同的散点，一种是（红色，小圆圈，z坐标取值范围(-50, -25)），另一种是（蓝色，小三角，z坐标取值范围(-30, -5)）。
for c, m, zlow, zhigh in [('r', 'o', -50, -25), ('b', '^', -30, -5)]:
    xs = randrange(n, 23, 32)
    ys = randrange(n, 0, 100)
    zs = randrange(n, zlow, zhigh)
    ax.scatter(xs, ys, zs, c=c, marker=m)

ax.set_xlabel('X Label')
ax.set_ylabel('Y Label')
ax.set_zlabel('Z Label')

plt.show()

# x=np.array([[1,0,0],[0,0.5,0],[0,0,1]])
# fig = plt.figure()
# ax = fig.add_subplot(1, 1, 1, projection='3d')
# for i in range(x.shape[0]):
#     ax.text(x[i, 0], x[i, 1], x[i, 2], str(1),
#             color='r',
#             fontdict={'weight': 'bold', 'size': 9})
# ax.view_init(20, 0)  # 只有这一行改变了
# plt.show()
# # 关键就是函数ax.view_init()函数的调用
# # 该函数接受两个参数，第一个参数是竖直旋转，第二个参数是水平旋转，旋转单位是度°
# # 默认的初始角度是ax.view_init(30, -60)



import setuptools
with open("README.md", "r",encoding='utf-8') as fh:
    long_description = fh.read()
setuptools.setup(
    name='difdyf',# 需要打包的名字,即本模块要发布的名字
    version='v1.14112',#版本
    description='A  module for diff geo', # 简要描述
    long_description=long_description,
    long_description_content_type="text/markdown",
    py_modules=[],   #  需要打包的模块
    packages=['','difdyf'],
    package_dir={'difdyf':'difdyf'},
    author='dyf', # 作者名
    author_email='715017912@qq.com',   # 作者邮件
    url='https://github.com/yftheone1995/dyf@riec.tohoku.ac.jp', # 项目地址,一般是代码托管的网站
    requires=['numpy','sympy'], # 依赖包,如果没有,可以不要
    license='MIT'
)


# import matplotlib.pyplot as plt
# import numpy as np
#
# fig, ax = plt.subplots()
# x = np.linspace(0, 10, 1000)
# y = np.sin(x)
# line = ax.plot(x, y)[0]
#
#
# def on_key_press(event):
#     # print(event.name)
#     print(event.button)
#     print(type(event.button))
#     print(event.button is 1)
#     print(event.button == 1)
#
# fig.canvas.mpl_disconnect(fig.canvas.manager.key_press_handler_id) # 取消默认快捷键的注册
# fig.canvas.mpl_connect('button_press_event', on_key_press)
# plt.show()
import sys, argparse
import math
import numpy as np
import matplotlib.pyplot as plt
import matplotlib.animation as animation
from scipy.spatial.distance import squareform, pdist, cdist
from numpy.linalg import norm

width, height = 640, 480

class Boids:
    """class that represents Boids simulation"""
    def __init__(self, N):
        """initialize the Boid simulation"""
        # initial position and velocities
        self.pos = [width/2.0, height/2.0] + 10*np.random.rand(2*N).reshape(N, 2)
        # normalized random velocities
        angles = 2*math.pi*np.random.rand(N)
        self.vel = np.array(list(zip(np.sin(angles), np.cos(angles))))
        self.N = N
        # minimum distance of approach
        self.minDist = 25.0
        # maximum magnitude of velocities calculated by "rules"
        self.maxRuleVel = 0.03
        # maximum magnitude of the final velocity
        self.maxVel = 2.0

    def tick(self, frameNum, pts, beak):
        """Update the simulation by one time step."""
        # get pairwise distances
        self.distMatrix = squareform(pdist(self.pos))
        # apply rules:
        self.vel += self.applyRules()
        self.limit(self.vel, self.maxVel)
        self.pos += self.vel
        self.applyBC()
        # update data
        pts.set_data(self.pos.reshape(2*self.N)[::2],
                     self.pos.reshape(2*self.N)[1::2])
        vec = self.pos + 10*self.vel/self.maxVel
        beak.set_data(vec.reshape(2*self.N)[::2],
                      vec.reshape(2*self.N)[1::2])

    def limitVec(self, vec, maxVel):
        """limit the magnitide of the 2D vector"""
        mag = norm(vec)
        if mag > maxVel:
            vec[0], vec[1] = vec[0]*maxVel/mag, vec[1]*maxVel/mag

    def limit(self, X, maxVel):
        """limit the magnitide of 2D vectors in array X to maxValue"""
        for vec in X:
            self.limitVec(vec, maxVel)

    def applyBC(self):
        """apply boundary conditions"""
        deltaR = 2.0
        for coord in self.pos:
            if coord[0] > width + deltaR:
                coord[0] = -deltaR
            if coord[0] < - deltaR:
                coord[0] = width + deltaR
            if coord[1] > height + deltaR:
                coord[1] = - deltaR
            if coord[1] < - deltaR:
                coord[1] = height + deltaR

    def applyRules(self):
        # apply rule #1: Separation
        D = self.distMatrix < 25.0
        vel = self.pos * D.sum(axis=1).reshape(self.N, 1) - D.dot(self.pos)
        self.limit(vel, self.maxRuleVel)

        # distance threshold for alignment (different from separation)
        D = self.distMatrix < 50.0

        # apply rule #2: Alignment
        vel2 = D.dot(self.vel)
        self.limit(vel2, self.maxRuleVel)
        vel += vel2

        # apply rule #3: Cohesion
        vel3 = D.dot(self.pos) - self.pos
        self.limit(vel3, self.maxRuleVel)
        vel += vel3

        return vel

    def buttonPress(self, event):
        """event handler for matplotlib button presses"""
        # print('run')
        # left-click to add a boid
        if event.button == 1:
            print('left button')
            self.pos = np.concatenate((self.pos, np.array([[event.xdata, event.ydata]])), axis=0)
            # generate a random velocity
            angles = 2*math.pi*np.random.rand(1)
            v = np.array(list(zip(np.sin(angles), np.cos(angles))))
            self.vel = np.concatenate((self.vel, v), axis=0)
            self.N += 1
        # right-click to scatter boids
        elif event.button == 3:
            print('right button')
            # add scattering velocity
            self.vel += 0.1*(self.pos - np.array([[event.xdata, event.ydata]]))


def tick(frameNum, pts, beak, boids):
    # print frameNum
    """update function for animation"""
    boids.tick(frameNum, pts, beak)
    return pts, beak

# main() function
def main():
    # use sys.argv if needed
    print('starting boids...')

    parser = argparse.ArgumentParser(description="Implementing Craig Reynold's Boids...")
    # add arguments
    parser.add_argument('--num-boids', dest='N', required=False)
    args = parser.parse_args()

    # set the initial number of boids
    N = 100
    if args.N:
        N = int(args.N)

    # create boids
    boids = Boids(N)

    # set up plot
    fig = plt.figure()
    ax = plt.axes(xlim=(0, width), ylim=(0, height))

    pts, = ax.plot([], [], markersize=10, c='k', marker='o', ls='None')
    beak, = ax.plot([], [], markersize=4, c='r', marker='o', ls='None')


    # add a "button press" event handler
    fig.canvas.mpl_connect("button_press_event", boids.buttonPress)
    anim = animation.FuncAnimation(fig, tick, fargs=(pts, beak, boids), interval=50)
    plt.show()

# call main
if __name__ == '__main__':
    main()



from sympy import *
from difdyf.util_func import *
from difdyf.plotfig import *
import numpy as np

########### r theta phi      3-d sphere axis
n=3
[ginput, inputsym]=sphereMetric()
####################################################### init the metric field  ginput

T=getChristSymb(ginput,inputsym)
print(T)          #############################################get the christ symbol
T_dict=list2Dict(T,inputsym,listDepth(T))
R=getRiemannCurvature(T,inputsym) #####################################get the riemann curvature tensor
R_dict=list2Dict(R,inputsym,listDepth(R))
print(R_dict)
rici=getRici(T,inputsym)
print(rici)
T2=(np.array(T)[1:3,1:3,1:3]).tolist()
print(getRiemannCurvature(T2,inputsym[1:3]))



coord=['r*cos(theta)*cos(phi)','r*cos(theta)*sin(phi)','r*sin(theta)']
var=['r','theta','phi']
plotFromCoord(coord,var,[[1],np.linspace(-1.57,1.57,40),np.linspace(0,6.28,40)])
